%% Demonstration of the application of the Barnett-Preisendorfer version 
%% of Canonical Correlation Analysis (CCA-BP) to the problem of
%% reconstructing temperature fields over the last millennium in the   
%% context of pseudoproxy experiments.  This demo uses a single set   
%% of parameters for pseduoproxy exeperiments targeting the temperature  
%% field of the NCAR CCSM 1.4 millennial simulation over a subdomain of the
%% NH as described in the following paper:
%% Smerdon, J.E., A. Kaplan, D. Chang, and M.N. Evans, 2010: 
%% A pseudoproxy evaluation of the CCA and RegEM methods for reconstructing 
%% climate fields of the last millennium, J. Climate, in press.
%% http://www.ldeo.columbia.edu/~jsmerdon/papers/2010a_jclim_smerdonetal.pdf
%%
%% Please note: in the code below the following assumptions about 
%% input data are made and used: 
%% (1) all grids (X, Y, Tyear) are uniform with positive increments; 
%% (2) Tyear grid increment is 1;
%% (3) the reconstruction interval preceeds immediately the calibration 
%%         interval; 
%% (4) longitude range definition used in locsNH(:,1) is consistent with that 
%%         used for X grid values: [-180, 180] longitude range is used for 
%%         both arrays.
%%
%% Reads the following files from ./data/ directory: 
%% ccsm_c180.mat, locsNH.mat, ccsm_pprox??.dat
%% 
%% Calls the following functions:
%% predccabp.m - computes CCA parameters necessary for reconstruction;
%% mapsubdom.m - plotting maps in geographical domain (uses imagescnan.m 
%%               by Carlos Adrian Vargas Aguilera);
%% lowpass.m   - lowpass filter provided within their packages of publicly
%%               available codes by the authors of Mann et al. [2005, 2007];
%%
%% Opens five figures and saves workspace into a file with the name 
%% ./CCArecon_*.mat that depends on user's runtime input (PNV, dp, dt, 
%% and dcca parameter values) 

 
%% J.E.Smerdon, A.Kaplan, 7 August 2010 
%%

clear all
close all

%load CCSM 1.4 surface temperature fields
load data/ccsm_c180.mat

% Please note for this script all grids (X, Y, Tyear) are assumed uniform 
% with positive increments; Tyear increment is assumed 1
nx=length(X);
ny=length(Y);
xw=X(1);
ys=Y(1);
dx=(X(nx)-xw)/(nx-1);
dy=(Y(ny)-ys)/(ny-1);

%specify the beginning and the end of the reconstruction period (years C.E.)
yrb=Tyear(1);
yre=1855;

%specify the beginning and the end of the calibration period (years C.E.)
ycb=1856;
yce=1980;

%compute these periods' lengths:
nr=yre-yrb+1;
nc=yce-ycb+1; 

% Please note: here we assume and use that the reconstruction interval 
% preceeds immediately the calibration interval. 
% Total length of reconstruction plus calibration time interval:
nt=nr+nc;

%reconstruction interval indices:
indr=1:nr;

%calibration interval indices:
indc=nr+1:nt;

% Please note: longitude range definition used in locsNH(:,1) is assumed 
% to be consistent with that used for X grid values. In the present paper 
% [-180, 180] range is used for both arrays.

% Load locations of model grids to be used
load data/locsNH.mat

% ns is a number of spatial locations to be reconstructed
[ns nd]=size(locsNH);

% find the smallest rectangular spatial domain 
% capturing the data to be reconstructed:

minloc = min(locsNH);
xwd = minloc(1)-dx/2;
ysd = minloc(2)-dy/2; 

maxloc = max(locsNH);
xed = maxloc(1)+dx/2;
ynd = maxloc(2)+dy/2;

% calculate area weights
weights0=cos(locsNH(:,2)*pi/180);
weights=weights0/sum(weights0);

% Convert grid coordinates locsNH to array indices inds: 
inds = zeros(ns,2);
inds(:,1)=round((locsNH(:,1)-xw)/dx+0.5);
inds(:,2)=round((locsNH(:,2)-ys)/dy+0.5);

% Extract model data at locations specified by locsNH for 
% reconstruction plus calibration temporal interval:
csmsub = nan(nt,ns);
for i = 1:ns
    csmsub(:,i) = tsurf(1:nt,inds(i,2),inds(i,1));
end

% constrain predictand to the calibration period, 
% compute matrix rank after centralization
csmsubc=csmsub(indc,:);
csmsubcd=csmsubc-ones(nc,1)*mean(csmsubc);
rankt=rank(csmsubcd);

display('For this demo, please select a desirable level') 
	    display('of noise in pseudoproxies in terms of')
	    display('signal-to-noise ratio (SNR) by standard deviation') 
	    display('or percent of noise variance (PNV) as follows:')
	    display('for SNR = infinity, PNV =  0, enter 1')
	    display('for SNR = 1,        PNV = 50, enter 2')
	    display('for SNR = 0.5,      PNV = 80, enter 3')
	    display('for SNR = 0.25,     PNV = 94, enter 4')
display('An appropriate file with precomputed pseudoproxies') 
display('will be taken from the data/ directory')
	    icase=input('Enter your selection here [1,2,3,4]: ');

	    s1='0589'; s2='0004';
	    sPNVarr=char(s1,s2)';
sPNV=sPNVarr(icase,:);

pnv=str2num(sPNV);
snr=sqrt(100/pnv-1);
sSNR=num2str(snr,'%4.2f');

pfilein=['data/ccsm_pprox' sPNV '.dat'];
prox=load(pfilein);

display(' ')

display(['Selected: PNV=' sPNV ', SNR=' sSNR])
display(' ')
display(['Pseudoproxy file named ' pfilein ' has been read']) 
% np is a number of proxies; note: ntp must be equal nt defined above
[ntp np]=size(prox);

% constrain predictors for the calibration period, 
% compute matrix rank after centralization
proxc=prox(indc,:);
proxcd=proxc-ones(nc,1)*mean(proxc);
rankp=rank(proxcd);

display(' ')
display('For suggested ("preferred") values of parameters dp, dt, and dcca')
display('please see Table 3 of the Smerdon et al. 2010 paper.')
display(' ')

dp = input(['Enter the value of dp [integer between 1 and ' num2str(rankp) ']: ']);

dt = input(['Enter the value of dt [integer between 1 and ' num2str(rankt)  ']: ']);

dccamax=min(dp,dt);
display(['Note that dcca dimension cannot exceed min(dp,dt), i.e., ' num2str(dccamax)])
dcca = input(['Enter the value of dcca [integer number between 1 and ' num2str(dccamax) ']: ']);

display(' ');

fileout=['CCArecon_' sPNV '_' num2str(dcca) '_' num2str(dp) '_' num2str(dt) '.mat'];
display ('Please note: the workspace at the end of this script will be'); 
display(['saved to the file named ' fileout]);

%start clock to measure the time elapsed for calculation
tic

%Perform the CCA reconstruction in the calibration period:
[Tp,D,Tm,Pm,Ts,Ps]=predccabp(proxc',csmsubc',dp,dt,dcca);

%Extend prediction to the full period:
drecon = (prox-ones(nt,1)*Pm')*diag(1.0./Ps)*D'*diag(Ts);
recon = drecon + ones(nt,1)*Tm';

% Calculate the weighted NH mean and the decadally filtered time series
% (centered on the calibration period mean)
reconm = drecon*weights;
reconmf = lowpass(reconm,0.1,1,1);

%Calculate the weighted NH mean of the model data
% (centered on the calibration period mean)
csmsubm = (csmsub-ones(nt,1)*Tm')*weights;
csmsubmf = lowpass(csmsubm,0.1,1,1);

% Calculate reconstruction period statistics:

%Calculate the NHMC
nhmc = corr(reconm(indr),csmsubm(indr));

%Calculate correlation field and its spatial mean
corrfield=nan(1,ns);
for i = 1:ns,
corrfield(i) = corr(recon(indr,i),csmsub(indr,i));
end
corrfieldm = corrfield*weights;

%Calculate the STD Ratios and their spatial mean
stdfield = std(recon(indr,:))./std(csmsub(indr,:));
stdfieldm = stdfield*weights;

%Calculate the mean bias and its spatial mean
d = recon(indr,:)-csmsub(indr,:);
biasfield = mean(d);
biasfieldm = biasfield*weights;

%Calculate the mean RMSE Field
mdiffsq = mean(d.^2);
rmsfield = sqrt(mdiffsq);
rmsw = sqrt(mdiffsq*weights);

t = toc;
display(' ')
disp(['Time elapsed (in seconds): ',sprintf(' %1.3f',t)])

casedesc = ['PNV=' sPNV ', SNR=' sSNR ', d_{cca}=' num2str(dcca) ', d_p=' num2str(dp) ', d_t=' num2str(dt) ];

%display results
display(' ')
display(['        Reconstruction interval (' num2str(yrb) '-' num2str(yre) ') statistics for']) 
display(['      the case of ' casedesc])
display('  NHMC   Mean Field Corr.   Mean STD Ratio   Mean Bias   Mean RMSE')
display([sprintf('  %4.2f         %4.2f              %4.2f           %4.2f        %4.2f',[nhmc corrfieldm stdfieldm biasfieldm rmsw])])
display(' ')

% Plot reconstruction period statistics fields

% Define tick intervals for X and Y axes to use in plots:
dxt=60;
dyt=20;

% plot correlation field
stitle = ['Correlation Field for ' casedesc];
ihf = mapsubdom(corrfield,inds,nx,ny,xw,ys,dx,dy,xwd,xed,ysd,ynd,dxt,dyt);
title(stitle)
caxis([0 1])
hc=get(ihf,'Children');
set(get(hc(1),'XLabel'),'String','Correlation Coefficient')

disp(stitle) 
display(['is plotted in Figure ' num2str(ihf)])
display(' ')

% plot std field
stitle = ['STD Ratio Field for ' casedesc];
ihf = mapsubdom(stdfield,inds,nx,ny,xw,ys,dx,dy,xwd,xed,ysd,ynd,dxt,dyt);
title(stitle)
caxis([0 1])
hc=get(ihf,'Children');
set(get(hc(1),'XLabel'),'String','Standard Deviation Ratio')

disp(stitle) 
display(['is plotted in Figure ' num2str(ihf)])
display(' ')

% plot bias field
stitle = ['Mean Bias Field for ' casedesc];
ihf = mapsubdom(biasfield,inds,nx,ny,xw,ys,dx,dy,xwd,xed,ysd,ynd,dxt,dyt);
title(stitle)
caxis([-.41 2])
hc=get(ihf,'Children');
set(get(hc(1),'XLabel'),'String','Mean Bias (K)')

disp(stitle) 
display(['is plotted in Figure ' num2str(ihf)])
display(' ')

% plot rmse field
stitle = ['RMSE Field for ' casedesc];
ihf=mapsubdom(rmsfield,inds,nx,ny,xw,ys,dx,dy,xwd,xed,ysd,ynd,dxt,dyt);
title(stitle)
caxis([0 3])
hc=get(ihf,'Children');
set(get(hc(1),'XLabel'),'String','RMSE (K)')

disp(stitle) 
display(['is plotted in Figure ' num2str(ihf)])
display(' ')

% plot NH mean time series and boxplot
ihf = figure;
subplot(2,2,[1 2])
stitle = ['Mean NH Time Series for ' casedesc];
plot(yrb:yce,reconmf,'r')
hold on
plot(yrb:yce,csmsubmf,'k')
axis([800 2000 -1.5 0.5])
title(stitle)
xlabel('Time (years C.E.)')
ylabel('Temperature Anomalies (K)')
grid on
legend1 = legend('CCA Recon. Mean','CSM Actual Mean');
set(legend1,'Position',[0.4637 0.6003 0.2438 0.08343]);
subplot(2,2,4)
boxplot([csmsubm(indr) reconm(indr)])
axis([.5 2.5 -2.5 0.5])
title('Box Plots for Mean NH Time Series')
xlabel('Model Target       CCA')
ylabel('Temperature Anomalies (K)')
grid on
set(get(gcf,'CurrentAxes'),'XTickLabel','')

display('Mean NH Time Series and Box Plots') 
display(['for ' casedesc]) 
display(['are in Figure ' num2str(ihf)])
display(' ')

clear t i s1 s2 d weights0 stitle hc
save(fileout)
%
